"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.upgrade = exports.getWorkspaces = exports.extractWorkspacesFromPkg = exports.readPkg = exports.resolveWorkspaces = exports.invoke = void 0;
var child_process_1 = __importDefault(require("child_process"));
var fs_1 = __importDefault(require("fs"));
var globby_1 = __importDefault(require("globby"));
var normalize_path_1 = __importDefault(require("normalize-path"));
var path_1 = __importDefault(require("path"));
var invoke = function (cwd, args, cmd) {
    var _a;
    var stdio = ['inherit', 'inherit', 'inherit'];
    var result = child_process_1.default.spawnSync(cmd, args, { cwd: (0, normalize_path_1.default)(cwd), stdio: stdio });
    if (result.error || result.status || result.signal !== null) {
        throw result;
    }
    return (_a = result.stdout) === null || _a === void 0 ? void 0 : _a.toString().trim();
};
exports.invoke = invoke;
var resolveWorkspaces = function (cwd, patterns) {
    var entries = globby_1.default.sync(patterns, {
        cwd: cwd,
        onlyFiles: false,
        absolute: true,
    });
    return Object.keys(entries.reduce(function (m, entry) {
        try {
            var stat = fs_1.default.lstatSync(entry);
            if (stat.isFile()) {
                m[path_1.default.dirname(entry)] = true;
            }
            if (stat.isDirectory()) {
                m[entry] = true;
            }
        }
        catch (_a) { }
        return m;
    }, {}));
};
exports.resolveWorkspaces = resolveWorkspaces;
var readPkg = function (cwd) {
    return JSON.parse(fs_1.default.readFileSync(path_1.default.resolve(cwd, 'package.json'), { encoding: 'utf8' }));
};
exports.readPkg = readPkg;
var extractWorkspacesFromPkg = function (pkg) {
    return pkg.workspaces
        ? Array.isArray(pkg.workspaces)
            ? pkg.workspaces
            : pkg.workspaces.packages
        : [];
};
exports.extractWorkspacesFromPkg = extractWorkspacesFromPkg;
var getWorkspaces = function (cwd, ws) {
    var pattern = typeof ws === 'string'
        ? ws.split(',')
        : Array.isArray(ws)
            ? ws
            : (0, exports.extractWorkspacesFromPkg)((0, exports.readPkg)(cwd));
    return (0, exports.resolveWorkspaces)(cwd, pattern);
};
exports.getWorkspaces = getWorkspaces;
var upgrade = function (cwd, flags) {
    var workspaces = __spreadArray([cwd], (0, exports.getWorkspaces)(cwd, flags.w || flags.workspaces), true);
    workspaces.forEach(function (ws) {
        console.log("invoke npm-upgrade for ".concat(ws));
        (0, exports.invoke)(ws, process.argv.slice(2), 'npm-upgrade');
    });
};
exports.upgrade = upgrade;
